@@ -93,7 +93,7 @@ sub new {
         exit 0;
     }
     # Look for the --mode argument first. If not found it is GCC
-    my $mode = 'GNUCC';
+    my $mode = $::default_mode;
     {
         my @args1 = ();
         foreach my $arg (@args) {
@@ -104,6 +104,7 @@ sub new {
             }
         }
         @args = @args1; # These are the argument after we extracted the --mode
+
     }
     if(defined $self->{MODENAME} && $self->{MODENAME} ne $mode) {
         die "Cannot re-specify the compiler";
@@ -133,6 +134,12 @@ sub new {
         foreach $key (keys %{$compiler}) {
             $self->{$key} = $compiler->{$key};
         }
+
+        # For MSVC we have to use --save-temps because otherwise the 
+        # temporary files get deleted somehow before CL gets at them !
+        if($mode ne "GNUCC" && $mode ne "AR") {
+            $self->{SAVE_TEMPS} = '.';
+        }
     }
 
     # Scan and process the arguments
@@ -352,6 +359,7 @@ Options:
   --includedir=xxx   Adds a new include directory to replace existing ones
   --stages           Show the processing stages
   --bytecode         Invoke the bytecode (as opposed to native code) system
+  --no-idashi        Do not use '-I-' with the gcc preprocessor.
 
 EOF
     $self->helpMessage();
@@ -1041,7 +1049,7 @@ sub doit {
 
     # See if we must create a library only
     if($self->{OPERATION} eq "TOLIB") {
-        $out = $self->linkOutputFile();
+        $out = $self->linkOutputFile(@tolink);
         $self->linktolib(\@tolink,  $out, 
                          $self->{PPARGS}, $self->{CCARGS}, 
                          $self->{LINKARGS});
@@ -1050,7 +1058,7 @@ sub doit {
 
     # Now link all of the files into an executable
     if($self->{OPERATION} eq "TOEXE") {
-        $out = $self->linkOutputFile();
+        $out = $self->linkOutputFile(@tolink);
         $self->link(\@tolink,  $out, 
                     $self->{PPARGS}, $self->{CCARGS}, $self->{LINKARGS});
         return;
@@ -1099,6 +1107,9 @@ sub compilerArgument {
               &{$action->{'RUN'}}($self, @fullarg, $onemore, $pargs);
               $argument_done = 1;
           }
+          # Quote special SHELL caracters 
+          @fullarg = map { $_ =~ s%([<>;&|])%'$1'%g; $_ } @fullarg;
+          # print "fullarg = ", @fullarg, "\n";
           if(defined $action->{'TYPE'}) {
               &classifyArgDebug("  type=$action->{TYPE}\n");
               if($action->{TYPE} eq 'EARLY_PREPROC') {
@@ -1168,8 +1179,23 @@ sub compilerArgument {
 sub runShell {
     my ($self, @cmd) = @_;
 
+    my $msvcFriends = 
+        ($self->{MODENAME} eq "MSVC" || 
+         $self->{MODENAME} eq "MSLINK" || 
+         $self->{MODENAME} eq "MSLIB");
+
     foreach (@cmd) {
 	$_ = $_->filename if ref;
+        # If we are in MSVC mode then we might have to convert the files
+        # from cygwin names to the actual Windows names
+        if($msvcFriends && $^O eq "cygwin") {
+            my $arg = $_;
+            if ($arg =~ m|^/| && -f $arg) { 
+                my $mname = `cygpath -m $arg`;
+                chop $mname;
+                if($mname ne "") { $_ = $mname; }
+            }
+        }
     }
 
     # sm: I want this printed to stderr instead of stdout
@@ -1238,6 +1264,16 @@ use strict;
 use File::Basename;
 use Data::Dumper;
 
+# For MSVC we remember which was the first source, because we use that to 
+# determine the name of the output file
+sub setFirstSource {
+    my ($self, $src) = @_;
+    
+    if(! defined ($self->{FIRST_SOURCE})) { 
+        $self->{FIRST_SOURCE} = $src;
+    }
+}
+
 sub new {
     my ($proto, $stub) = @_;
     my $class = ref($proto) || $proto;
@@ -1283,25 +1319,27 @@ sub new {
 # given subroutine is invoked with the self, the argument and the (possibly
 # empty) additional word and a pointer to the list of remaining arguments
 #
-          ["^[^/\\-@].*\\.($::cilbin|c|cpp|cc)\$" => { TYPE => 'CSOURCE' },
+          ["^[^/\\-@].*\\.($::cilbin|c|cpp|cc)\$" => 
+           { TYPE => 'CSOURCE',
+             RUN => sub { &MSVC::setFirstSource(@_); } },
            "[^/].*\\.(asm)\$" => { TYPE => 'ASMSOURCE' },
            "[^/].*\\.i\$" => { TYPE => 'ISOURCE' },
            "[^/\\-@]" => { TYPE => "OSOURCE" },
-           "/O" => { TYPE => "CC" },
+           "[/\\-]O" => { TYPE => "CC" },
            "[/\\-][DI]" => { TYPE => "PREPROC"},
-           "/EH" => { TYPE => "CC" },
-           "/G"  => { TYPE => "CC" },
-           "/F[aA]"  => { TYPE => 'CC' },
+           "[/\\-]EH" => { TYPE => "CC" },
+           "[/\\-]G"  => { TYPE => "CC" },
+           "[/\\-]F[aA]"  => { TYPE => 'CC' },
            "[/\\-]Fo"     => { TYPE => 'OUT' },
            "/Fe"   => { TYPE => 'OUT',
                         RUN => sub { $stub->{OPERATION} = "TOEXE" }},
            "[/\\-]F[dprR]" => { TYPE => "CC" },
            "[/\\-]FI" => { TYPE => "PREPROC" },
-           "/[CXu]" => { TYPE => "PREPROC" },
-           "/U" => { ONEMORE => 1, TYPE => "PREPROC" },
-           "/(E|EP|P)" => { RUN => sub { push @{$stub->{PPARGS}}, $_[1]; 
+           "[/\\-][CXu]" => { TYPE => "PREPROC" },
+           "[/\\-]U" => { ONEMORE => 1, TYPE => "PREPROC" },
+           "[/\\-](E|EP|P)" => { RUN => sub { push @{$stub->{PPARGS}}, $_[1]; 
                                          $stub->{OPERATION} = "PREPROC"; }},
-           "/c" => { RUN => sub { $stub->{OPERATION} = "TOOBJ"; }},
+           "[/\\-]c" => { RUN => sub { $stub->{OPERATION} = "TOOBJ"; }},
            "[/\\-](Q|Z|J|nologo|w|W|Zm)" => { TYPE => "CC" },
            "[/\\-]Y(u|c|d|l|X)" => { TYPE => "CC" },
            "[/\\-]T(C|P)" => { TYPE => "PREPROC" },
@@ -1312,9 +1350,9 @@ sub new {
                     $fname = &normalizeFileName($fname);
                     push @{$stub->{CFILES}}, $fname;
                 }},
-           "/v(d|m)" => { TYPE => "CC" },
-           "/F" => { TYPE => "CC" },
-           "/M"   => { TYPE => 'LINKCC' },
+           "[/\\-]v(d|m)" => { TYPE => "CC" },
+           "[/\\-]F" => { TYPE => "CC" },
+           "[/\\-]M"   => { TYPE => 'LINKCC' },
            "/link" => { RUN => sub { push @{$stub->{LINKARGS}}, "/link", 
                                           @{$_[3]};
                                      @{$_[3]} = (); } },
@@ -1433,10 +1471,13 @@ sub assembleOutputFile {
 
 sub linkOutputFile {
     my($self, $src) = @_;
+    $src = $src->filename if ref $src;
     if(defined $self->{OUTARG} && "@{$self->{OUTARG}}" =~ m|/Fe(.+)|) {
         return $1;
     }
-    return "a.exe";
+    # Use the name of the first source file, in the current directory
+    my ($base, $dir, $ext) = fileparse ($src, "\\.[^.]+");
+    return "./$base.exe";
 }
 
 sub setVersion {
@@ -1764,8 +1805,7 @@ sub new {
             "[^-]" => { RUN => sub { &GNUCC::parseLinkerScript(@_); }},
             "-E"   => { RUN => sub { $stub->{OPERATION} = "TOI"; }},
 	    "-pipe\$" => { TYPE => 'ALLARGS' },
-            "-[DI]" => { ONEMORE => 1, TYPE => "PREPROC" },
-            '-U$' => { TYPE => 'PREPROC', ONEMORE => 1 },
+            "-[DIU]" => { ONEMORE => 1, TYPE => "PREPROC" },
             '-undef$' => { TYPE => 'PREPROC' },
             '-w$' => { TYPE => 'PREPROC' },
 	    '-M$' => { TYPE => 'SPECIAL' },
